/* Main program written in C++ that calls two Fortran DLL routines */
/* This  program implements a Monte Carlo routine for
pricing financial products */
#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "stdafx.h"
#include <math.h>

#define ALPH 1.5
#define NDMX 50
#define MXDIM 10
#define TINY 1.0e-30

#define IM1 2147483563 
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)



long idum;	//for random number inizialization in main



/* Declare the Fortran routine. The following items are of note:
   - Fortran character arguments have a hidden length
     argument following the address
   - Routine name must be in uppercase to match Fortran.
*/
                        
 
 
double xxs[101], sg[101], rhoij, t;
 double rho[101][101];
  int nt;
   typedef void (CALLBACK* DLL_ROUT)(int *nv,
    double x[101],
        double sg[101],
        double xxs[101],
        double *rhoij,
        double *t,
        int *nt,
	    int *iii,
        int *jjj,
        double *value);            


 
typedef void (CALLBACK* DLL_ROUT12)(int *nv,
    double x[101],
        double sg[101],
        double xxs[101],
        double *t,
        int *nt,
        double *value);            
 
 void main (int argc, char *argv[])
{	
	void vegas(int ndim, double(*fxn)(int, double[]), 
                int ncall, double tgral, double sd);
	
	double fxn(int, double[]);
	
	
	int  ndim;
	double   sd;
	double tgral;
	int ncall;
     int i,j;

	 extern double xxs[101], sg[101], rhoij, t;
	 extern double  rho[101][101];
     extern int nt;



	double value=0;

// Assign ndim: number of spatial variables (Bound:1<ndim<102). The integer ndim is
//      the number of underlyings considered.      
   
	ndim=2;

// Assign t: time to maturity (Bound: 0<t<1) where the probability density must
//           be evaluated.
       
		t=0.5;

/*
!!   nt: number of terms in the time series expansion in time of the first order correction
!!       term (see formula (7.1) Section 7). The largest value of nt feasible is 9. The strongly
!!       recommended value is nt=2.
*/

        nt=2;
//
//  Assign the volatilities sg[0],sg[1],...,sg[ndim-1]. For i=0,1,...,ndim-1 sg[i] is the volatilty 
//   associated to the (i+1)-th underlying. Bound: 0.005<sg[i]<0.6, i=0,1,...,ndim-1.
//
		sg[0]=0.15;
	           sg[1]=0.1;
		for(i=2; i<ndim; i++)
                sg[i]=0.1;


// Assign xxs[], the vector containing the point whose components 
// are the prices at time to maturity t>0 where the financial derivative 
// must be evaluated. Bound: 0.005<xxs[i]<10, i=0,1,...,ndim-1.
// For i=0,1,...,ndim-1 xxs[i] is the price of the (i+1)-th underlying.
// Example:
     for(i=0; i<ndim/2; i++){
                xxs[i]=0.1-0.05;
				}
	    for(i=ndim/2; i<ndim; i++){
                xxs[i]=0.1+0.05;
				}
//
//  Assign the correlation matrix rho[i][j], i=0,1,...,ndim-2, j=i+1,i+2,...,ndim-1.
//  (Bound: -1<rho[i][j]<1, i=0,1,...,ndim-2, j=i+1,i+2,...,ndim-1).
// For i=0,1,...,ndim-2, j=i+1,i+2,,,,mndim-1 rho[i][j] is the correlation coefficient
// between the square roots of the (i+1)-th underlying and of the (j+1)-th underlying.
//
    for(i=0; i<ndim-1; i++)
      { for(j=i+1; j<ndim; j++)
            rho[i][j]=-0.06;
      }
// The remaining correlation coefficients are fixed as follows:
         rho[0][0]=1.0;
	    for(i=1; i<ndim; i++)
			rho[i][i]=1.0;
      { for(j=0; j<i; j++)
            rho[i][j]=rho[j][i];
      }

//
// ncall: number of function evaluations used by the Monte Carlo routine.
//
// hint: let ep=max_{1,2,...,ndim} sg[i-1]**2*t
//          then choose   ncall>=max(10000*ep*ndim, 50000).
//            
            ncall=5000;
// call Monte Carlo routine:
//
   		
//
//tgral: estimated value of the integral.
//   sd: estimated error in the integral approximation tgral.
		
		// Initialize vegas variables
	
	tgral=0;
	sd=0;
	

	vegas(ndim, *fxn, ncall, tgral, sd);
}





void vegas(int ndim, double(*fxn)(int,double[]), int ncall,
		   double tgral, double sd)
//
//  This routine computes the integral of the function defined in fxn 
//  on [0, +infinity]**ndim
//
//
//  ndim: number of spatial variables (1<ndim<102).
//  fxn        : function to be integrated.
//  ncall      : number of function evaluations used to compute the integral.


{
	double ran2(long *idum);
        static int i, it, j, k,  mds, nd, ndo, ng, npg;
        static double calls, dv2g, dxg, f, f2, f2b, fb;
    	static double x[101];
        static double app[101], gauss[101], sgt[101], radxs[101];
        static double tgralp, sdp, fmed, var, varp;
		static double radt2, www, unif1, unif2, value;
		
	 extern double xxs[101], sg[101], rhoij, t;
     extern double rho[101][101];
     extern int nt;
		long icont,idum;


        sd=0.0;
        tgral=0.0;
        fmed=0.0;
        var=0.0;
        radt2=sqrt(t)/2.0;
    for(i=0; i<ndim; i++){
         radxs[i]=sqrt(xxs[i]);
         sgt[i]=radt2*sg[i];
         app[i]=-2.0*radxs[i]/(sg[i]*sqrt(t));
		   }
        icont=1;
        for(i=0; i<ncall/2; i++){
			
// Write the value of integral obtained and the relative error when using "2i" random points.
        tgralp= fmed/(2.0*(double)(i+1));
        varp=var/(2.0*(double)(i+1));
        sdp=sqrt((varp-tgralp*tgralp)/(2.0*(double)(i+1)));

          printf("%6d %f %f\n", i, tgralp, sdp);
        idum=i;

//
// ran2() generates a random number uniformly distributed on (0,1)
//

        for(j=0; j<ndim; j++){
	       www=2.0;
			   while( www>1.0) {
          icont=icont+1;
/*!
!! Generator of normally distributed random numbers with probability density: 
!!  p(z)=exp(-z^2/2)/sqrt(Pi) where  Pi=arcos(-1)
!
! Polar Marsilia random numbers generator.
!
*/
              unif1=ran2(&icont);
              icont=icont+1;
              unif2=ran2(&icont);
			  
              unif1=2.0*unif1-1.0;
              unif2=2.0*unif2-1.0;
			 
              www=unif1*unif1+unif2*unif2; 
		   }
             
   
              gauss[2*j-1]=sqrt(-2.0*log(www)/www)*unif1;
              gauss[2*j]=sqrt(-2.0*log(www)/www)*unif2;

 
              x[2*j-1]=(radxs[2*j-1]+sgt[2*j-1]*gauss[2*j-1]);
              x[2*j-1]=x[2*j-1]*x[2*j-1];
              x[2*j]=(radxs[2*j]+sgt[2*j]*gauss[2*j]);
              x[2*j]=x[2*j]*x[2*j];
          }

        for(j=0; j<ndim; j++){
		   if(app[j]>gauss[j]){
            value=0.0;
            goto errorone;}
            }
             
 
         value=fxn(ndim,x);
  errorone:
          fmed=fmed+value;
          var=var+value*value;
//
// antithetic variables:
//
        for(j=0; j<ndim; j++){
                 gauss[j]=-gauss[j];
              x[j]=(radxs[j]+sgt[j]*gauss[j]);
              x[j]=x[j]*x[j];
           }
        for(j=0; j<ndim; j++){
			if(app[j]>gauss[j]){
            value=0.0;
            goto errortwo;}
            }
          value=fxn(ndim,x);
errortwo:
           fmed=fmed+value;
          var=var+value*value;
        }


// ! closure loop on i=1,2,..,ncall/2.


        fmed=fmed/(double)ncall;
        tgral=fmed;
        var=var/(double)ncall;
        sd=sqrt((var-fmed*fmed)/(double)ncall);
        printf("ncall= %6d \n integral value= %f \n estimated error %8f\n", ncall, tgral, sd);
        
}        

		
// Function: fxn.

double fxn(int ndim,double x[]){

      extern  double sg[101],  xxs[101],  rhoij,  t; 
    extern double rho[101][101];
  	extern int nt;

	double p0,p1, kprice,aux;
		int i,j;

		

        double value;
	int nv;
    nv=ndim;    
    value = 0;           
   int  iii, jjj;




    HINSTANCE hdll, hdll12;
	  DLL_ROUT pointf;
	   DLL_ROUT12 pointf12;


			hdll12=LoadLibrary("UnivPM_MathFinSolver1_2.dll");
	if(!hdll12)
	{ printf("Read Error");
	value=-100;
	goto errore;
	}
	if(!(pointf12 = (DLL_ROUT12)GetProcAddress(hdll12,"DLL_ROUT12")))
	{ printf("error");
	value=-100;
	goto errore;
	}



 
//  Zero-th order term.
//
// Call dll UnivPM_MathFinSolver1_2:
//

	 pointf12(&nv,x,sg,xxs,&t, &nt, &value);
	   p0=value;
	    FreeLibrary(hdll12);

			   		hdll=LoadLibrary("UnivPM_MathFinSolver1_3.dll");
	if(!hdll)
	{ printf("Read Error");
	value=-100;
	  goto errore;
	}
	if(!(pointf = (DLL_ROUT)GetProcAddress(hdll,"DLL_ROUT")))
	{ printf("error");
	value=-100;
	 goto errore;
	}
//
//  First order term.
//

       p1=0.0;
	   if(nt<0) goto jump;
       for(iii=0; iii<nv-1; iii++)
	   {
          		   for(jjj=iii+1; jjj<nv; jjj++)
		   {

//
// Call dll UnivPM_MathFinSolver1_3:
//
		  i=iii+1;
		  j=jjj+1;
		  rhoij=rho[iii][jjj];
           pointf(&nv,x,sg,xxs,&rhoij,&t,&nt,&i, &j,&value);
           p1=p1+value;
		   }
	   }
	
jump:
     value=p1+p0;
//
// value contains the first order approximation of the transition probability
//      density.
//

//	 
// aux: payoff function (to be assigned).
//
// In the example aux=max(min(x[i])-kprice,0), i=0,1,..,nv-1.
//
	 kprice=0.1;
	 aux=x[0]-kprice;
	 for(i=1;i<nv;i++){
		 if(aux<(x[i]-kprice)) aux=x[i]-kprice;
	 }
	 if(aux<0.0) aux=0.0;
//
// to test the parameters nt, ncall set aux=1.0. The value of the integral
// must be one. Remove "comment" from the following instruction.
//   aux=1.0

     value=value*aux;
errore:
	

   FreeLibrary(hdll);
     

return(value);
}





double ran2 (long *idum){
	int j;
	long k;
	static long idum2=123456789;
	static long iy=0;
	static long iv[NTAB];
	double temp;

	if(*idum<=0){
		if(-(*idum)<1) *idum=1;
		else *idum=-(*idum);
		idum2=(*idum);
		for(j=NTAB+7;j>=0;j--){
			k=(*idum)/IQ1;
			*idum=IA1*(*idum-k*IQ1)-k*IR1;
			if(*idum<0) *idum += IM1;
			if(j<NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ1;
	*idum=IA1*(*idum-k*IQ1)-k*IR1;
	if(*idum<0) *idum += IM1;
	k=idum2/IQ2;
	idum2=IA2*(idum2-k*IQ2)-k*IR2;
	if(idum2<0) idum2 += IM2;
	j=iy/NDIV;
	iy=iv[j]-idum2;
	iv[j]= *idum;
	if(iy<1) iy += IMM1;
	if((temp=AM*iy)>RNMX) return RNMX;
	else return temp;
}
