! Main program written in Fortran that calls two Fortran DLL routines.
! This program implements a Monte Carlo routine for pricing financial products.
! The following items are of note:
! - The dll import specification is required
! - Routine name must be in uppercase to match Fortran.
!
! Declare an interface block to the routine to be called.
	interface
	  subroutine rtn_DLL_ROUT(nv,x,sg,xxs,rhoij,t,nt,iii,jjj,value)	   	 
		real*8 xxs(101), x(101), sg(101), rhoij, t, value
		integer nv, nt,iii,jjj
	  end subroutine
	  	  subroutine rtn_DLL_ROUT12(nv,x,sg,xxs,t,nt,value)	   	 
		real*8 xxs(101), x(101), sg(101), t, value
		integer nv, nt
	  end subroutine


	end interface

	call beginning()
end


subroutine beginning()
  integer  ndim, nt, nv
  real*8  resf,  value		
  integer i, j, iii, jjj
  real*8 x(101),rho(101,101), sg(101), xxs(101), t
  real*8 z(101), min
  real*8 fxn, sd, tgral
  integer*4 ncall 
  common/areadati/rho,sg,xxs,t
  external fxn

! Assign ndim: number of spatial variables (Bound:1<ndim<102). The integer ndim is
!       the number of underlyings considered.
	    ndim=2
		if (ndim.gt.102) then 
			write(*,*) 'Error: insert the variables number. must be greater than 1 and less than 102'
			stop
		endif

! Assign t: time to maturity (Bound: 0<t<1) where the probability density must
!       be evaluated.

		t=0.5

                if(t.Ge.1.0d0.OR.t.LE.0.0) then
			write(*,*) 'Error in time to maturity t: 0<t<1'
			stop
		endif

!  Assign the volatilities sg(1),sg(2),...,sg(ndim). Bound: 0.005<sg(i)<0.6, i=1,2,...,ndim.
               sg(1)=0.15
	           sg(2)=0.1
                 do i=3, ndim
                   sg(i)=0.1
                 enddo
				 

!  Assign the correlation matrix rho(i,j), i=1,2,...,ndim-1, j=i+1,i+2,...,ndim (Bound: -1<rho(i,j)<1, i=1,2,...,ndim-1, j=i+1,i+2,...,ndim).
                 do i=1, ndim-1
                   do j=i+1, ndim
                     rho(i,j)=-0.06
                   enddo
                 enddo
! The remaining correlation coefficients are fixed as follows:

                 rho(1,1)=1.d0
                 do i=2, ndim
                   rho(i,i)=1.d0
                   do j=1, i-1
                     rho(i,j)=rho(j,i)
                   enddo
                 enddo
! Assign xxs(), the vector containing the point whose components 
! are the prices at time to maturity t>0 where the financial derivative 
! must be evaluated. Bound: 0.005<xxs(i)<10, i=1,2,...,ndim.
! Example:
!     

                 do i=1, ndim/2
                   xxs(i)=0.1-0.05
                 enddo
                 do i=1+ndim/2,ndim
                   xxs(i)=0.1+0.05
                 enddo


      
       
!
!  ncall: number of function evaluations used by the Monte Carlo routine.
!
! hint: let ep=max_{1,2,...,ndim} sg(i)**2*t
!           then choose   ncall>=max(10000*ep*ndim, 50000).
!            
            ncall=5000
! call Monte Carlo routine:
!!!
      call vegas(ndim, fxn,  ncall,tgral,sd)
!!!
! tgral: estimated value of the integral.
!    sd: estimated error in the integral approximation tgral.


             write(*,*) 'value integral:', tgral
             write(*,*) 'standard deviation', sd
             write(15,*) 'time=',t
             write(15,*) 'value integral:', tgral
             write(15,*) 'standard deviation', sd
             write(15,*) 'function evaluations', ncall

      end

!
!!!! Function to be integrated
!

                 real*8 function fxn(ndim,x)
                 integer ndim
                 real*8 x(101)
                 real*8 sg(101), rho(101,101), xxs(101)
                 real*8 resf, t, kprice, aux

                 common/areadati/rho,sg,xxs, t
!     
!
!! dll parameters:
!!   nv: number of spatial variables  (nv=ndim).
!!  x(): integration variables (x is an  array of dimension ndim, assigned by the Monte Carlo routine).
!!xxs(): the vector containing the point whose components are the prices at time to maturity t>0 where the 
!!       transition probability density must be evaluated.
!!   sg: array of dimension nv containing the volatilities (assigned by the user in the main).
!!  rho: correlation matrix of dimension nv x nv (assigned by the user in the main)
!!    t: time to maturity where the transition probability density must be evaluated.
!!       (assigned by the user in the main).
!!   nt: number of terms in the time series expansion in time of the first order correction
!!       term (see formula (7.1) Section 7). The largest value of nt feasible is 9. The strongly
!!       recommended value is nt=2.

                 nt=2
                 nv=ndim
                 
! call dll:
                 call calcp(nv,x,sg,xxs,rho,t, nt, resf)
!
! resf: value returned by the dll that computes the transition probability
!       density.
!
! aux: payoff function (to be assigned).
!
! In the example aux=max(min(x(i))-kprice,0).
!
                 kprice=0.1
                 aux=x(1)-kprice
                 do i=2, nv
                   if(aux.lt.(x(i)-kprice)) aux=x(i)-kprice
                 enddo
                 if(aux.lt.0.0) aux=0.0
!
! to test the parameters nt, ncall  set aux=1. The value of the integral in this case
! must be one. Remove the "comment" from the following instruction.
!!
!                 aux=1
!!! 
                 fxn=resf*aux
                 return
                 end




!!!!!!

      subroutine vegas(ndim, fxn,  ncall, tgral,sd)
        
!
!  This routine computes the integral of the function defined in fxn 
!  on [0, +infinity]**ndim
!
!
!  ndim: number of spatial variables (1<ndim<102).
!  fxn        : function to be integrated.
!  ncall      : number of function evaluations used to compute the integral.

       
         real*8  tgral, sd, chi2a,fmed,vol,var
         real*8 value
         integer ndim
         integer*4 ncall, i, icont
         real*8 fxn
         external fxn
         real*8 ran2, unifrd
         external ran2
         external unifrd
         integer idum
!
!! vegas recalls  float ran2(long idum);
!
        integer it, j, k
        real*8 calls, dv2g, dxg, f, f2, f2b, fb, ti, tsi
        real*8 wgt, xjac, xn, xnd, xo
        real*8 xxs(101)
        real*8 x(101),rho(101,101), sg(101), t, pi2
        real*8 app(101), www,sum, sdp, varp, tgralp
        real*8 unif1, unif2, gauss(101), radt2,radxs(101), sgt(101)

        common/areadati/rho,sg,xxs,t

        real*8 schi, si, swgt
!!
!!! The points are normally distributed.
!!
        sd=0.d0
        tgral=0.d0
        fmed=0.d0
        var=0.d0
        pi2=2.d0*dacos(-1.d0)
        radt2=dsqrt(t)/2.d0
        do i=1, ndim
         radxs(i)=dsqrt(xxs(i))
         sgt(i)=radt2*sg(i)
         app(i)=-2.d0*radxs(i)/(sg(i)*dsqrt(t))
        enddo
        icont=0
        do i=1, ncall/2
!! Write the value of integral obtained and the relative error when using "2i" random points.
        tgralp= fmed/(2.0*dfloat(i))
        varp=var/(2.0*dfloat(i))
        sdp=dsqrt((varp-tgralp**2)/dfloat(2*i))
!! 
	  write(*,*) i, tgralp, sdp

        idum=i-1

!
!! ran2() generates a random number uniformly distributed on (0,1).
!

         do j=1, ndim/2
	       
222           icont=icont+1

!
!! Generator of normally distributed random numbers with probability density: 
!!  p(z)=exp(-z^2/2)/sqrt(Pi) where  Pi=arcos(-1)
!
! Polar Marsilia random numbers generator.
!
155              unif1=ran2(icont)
              icont=icont+1
              unif2=ran2(icont)
              unif1=2.0*unif1-1.d0
              unif2=2.0*unif2-1.d0
              www=unif1*unif1+unif2*unif2
              if(www.gt.1.0) go to 222
              gauss(2*j-1)=dsqrt(-2.d0*dlog(www)/www)*unif1
              gauss(2*j)=dsqrt(-2.d0*dlog(www)/www)*unif2


              x(2*j-1)=(radxs(2*j-1)+sgt(2*j-1)*gauss(2*j-1))**2
              x(2*j)=(radxs(2*j)+sgt(2*j)*gauss(2*j))**2

         enddo
		    if((ndim-2*(ndim/2)).ne.0) then
223              icont=icont+1
              unif1=ran2(icont)
              icont=icont+1
              unif2=ran2(icont)
              unif1=2.0*unif1-1.d0
              unif2=2.0*unif2-1.d0
              www=unif1*unif1+unif2*unif2
              if(www.gt.1.0) go to 223
              gauss(ndim)=dsqrt(-2.d0*dlog(www)/www)*unif1
              x(ndim)=(radxs(ndim)+sgt(ndim)*gauss(ndim))**2
         endif
     
       do j=1, ndim
          if(app(j).gt.gauss(j)) then
            value=0.d0
            go to 444
          endif
       enddo

          value=fxn(ndim,x)

444          fmed=fmed+value
          var=var+value**2
!
!!  antithetic variables:
!
              do j=1, ndim
                 gauss(j)=-gauss(j)
                 x(j)=(radxs(j)+sgt(j)*gauss(j))**2
             enddo
       do j=1, ndim
          if(app(j).gt.gauss(j)) then
            value=0.d0
            go to 445
          endif
       enddo


          value=fxn(ndim,x)
445     continue

           fmed=fmed+value
          var=var+value**2

667     continue
        enddo
! closure loop on i=1,2,..,ncall/2.

        fmed=fmed/dfloat(ncall)
        tgral=fmed
        var=var/dfloat(ncall)
        sd=dsqrt((var-fmed**2)/dfloat(ncall))

        return
        end
!!!
!!! Closure Vegas.
!!!
!!
      real*8 function ran2(idum)
      real*8 rm
      PARAMETER (M=714025,IA=1366,IC=150889,RM=1.4005112E-6)
      DIMENSION IR(97)
        save
      DATA IFF /0/
      IF(IDUM.LT.0.OR.IFF.EQ.0)THEN
        IFF=1
        IDUM=MOD(IC-IDUM,M)
        DO 11 J=1,97
          IDUM=MOD(IA*IDUM+IC,M)
          IR(J)=IDUM
11      CONTINUE
        IDUM=MOD(IA*IDUM+IC,M)
        IY=IDUM
      ENDIF
      J=1+(97*IY)/M
      IF(J.GT.97.OR.J.LT.1)PAUSE
      IY=IR(J)
      RAN2=IY*RM
      IDUM=MOD(IA*IDUM+IC,M)
      IR(J)=IDUM
      RETURN
      END

!! Subroutine that computes the first order approximation of p.


subroutine calcp(nv,x,sg,xxs,rho,t,nt,res)
!use dfwin
	use kernel32
!dll variables
!!        nv: number of variables.
!!     x(nv): integration variable.
!!   xxs(nv): vector containing the point where the financial product must be evaluated.
!!    sg(nv): vector containing the volatilities. sg(i) is
!!            the volatility of the i-th underlying.
!!rho(nv,nv): rho_{i,j} is the correlation coefficient matrix.
!!         t: time to maturity.
!!        nt: integer representing the number of terms of the power series expansion
!!               in time to maturity of p_1 used to approximate p_1.
!!     value: approximated value of the transition probability density funcion p.

    real*8 xxs(101), x(101), sg(101), rho(101,101), t, value, res, p0,p1
    integer nv, nt, jjj, iii
     real*8 rhoij      

!	 
!to call dll:
	pointer (p,i)
	pointer (q,rtn_DLL_ROUT)
	logical status

	!to call dll
	pointer (pp,i1)
	pointer (qq,rtn_DLL_ROUT12)
	logical status2


! First, locate the dll and load it into memory.
    p = loadlibrary("UnivPM_MathFinSolver1_3.dll"C)
	if (p == 0) then
		type *, "Error occurred opening testsub.dll"
		type *, "Program aborting"
		goto 1000
	endif

! Set up a pointer to the routine of interest.
    q = getprocaddress(p, "DLL_ROUT"C)
	if (q == 0) then
		type *, "Error occurred finding DLL_ROUT in testsub.dll"
		type *, "Program aborting"
		goto 1000
	endif

	! First, locate the dll and load it into memory.
    pp = loadlibrary("UnivPM_MathFinSolver1_2.dll"C)
	if (pp == 0) then
		type *, "Error occurred opening testsub2.dll"
		type *, "Program aborting"
		goto 1000
	endif

! Set up a pointer to the routine of interest.
    qq = getprocaddress(pp, "DLL_ROUT12"C)
	if (qq == 0) then
		type *, "Error occurred finding DLL_ROUT in testsub2.dll"
		type *, "Program aborting"
		goto 1000
	endif


!
!!  Zero-th order term.
!
! Call dll UnivPM_MathFinSolver1_2:
	value=0
  call rtn_DLL_ROUT12(nv,x,sg,xxs,t,nt,value)
  status = freelibrary(pp)
  p0=value


!!  First order term.
!
       p1=0.d0
	   if(nt.lt.0) go to 1000
       do  iii=1, nv-1
          do  jjj=iii+1, nv
		  
!
! Call dll UnivPM_MathFinSolver1_3:
!
            value=0.0
			rhoij=rho(iii,jjj)
            call rtn_DLL_ROUT(nv,x,sg,xxs,rhoij,t,nt,iii,jjj,value)
		
           p1=p1+value
	    enddo
	 enddo
	 status = freelibrary(p)
1000 continue
     res=p1+p0

        return

       end











	 
